#!/bin/sh
#
# Check if there is any HL7 device configured locally and send
# the HL7 message them if they support it.
#
# Also proliferate the HL7 message in the dotcom
#
# Usage:
#    sendMessage.sh -f <filename> [-n] [-e <event>] [-d <direction>] [-x <excepthost>]
#
#        -f <filename>   : HL7 message file to be sent and proliferated
#        -n              : do not proliferate HL7 message to Database servers
#        -e <event>      : Event triggered the creation of HL7 message
#        -x <excepthost> : do not proliferate the file to this host
#        -d <direction>  : direction of proliferation {up|down|all}, default is all
#

. $HOME/etc/setenv.sh

LOGGER="/usr/bin/logger"
TAG="sendHL7Message[$$]"

#
# Usage
#
function usage_()
{
	if [ -n "$1" ] ; then
		echo $1
		$LOGGER -p $LOG_FACILITY.err -t "$TAG" "$1"
	fi
	cat <<- ENDOFUSAGE
		sendMessage.sh -f <filename> [-e event] [-o <device>] [-d <direction>] [-x <excepthost>]
	
		 -f <filename>   : HL7 message file to be sent and proliferated
		 -e <event>      : Event triggered the creation of HL7 message
		 -o <device>     : Originating device (default: LOCAL)
		 -x <excepthost> : do not proliferate the file to this host
		 -d <direction>  : direction of proliferation {up|down|all}, default is all
	ENDOFUSAGE
} # usage


# proliferate.sh can be called directly:
# run as a task instead
if [ `basename $0` = "sendMessage.sh" ] ; then
	$TASKD_HOME/runshellcmd.sh -R -r 0 $0 "$@"
	exit $?
fi

ERRMSG=""
FILENAME=""
EVENT="unrecognized"
DEVICE="LOCAL"
DIRECTION="all"
EXCEPT=""

EXCLUDE_DBONLY=""

while getopts 'f:e:o:x:d:n' OPT; do
	case $OPT in
		f) FILENAME="$OPTARG" ;;
		e) EVENT="$OPTARG" ;;
		o) DEVICE="$OPTARG" ;;
		x) EXCEPT="$OPTARG" ;;
		d) DIRECTION="$OPTARG" ;;
		n) EXCLUDE_DBONLY="-n" ;;
		*) ERRMSG="Invalid_Parameter:$OPT" ;;
	esac
done

[ -z "$FILENAME" ] && ERRMSG="Missing Parameter"

rcid=`basename ${FILENAME}`

#
# Checks if an event is supported by a device
#
# $1 - device
#
function isEventSupported_()
{
	[ -z "$1" ] && return 1

	for cfg in $IFACE_ORDER_CONFIG_FILE $IFACE_REPORT_CONFIG_FILE ; do
		[ ! -f "$cfg" ] && continue

		RES=`awk -vDevSrc="$DEVICE" -vDevDest="$1" -vEvent="$EVENT" ' \
			($1 == DevSrc) { \
			 if (Event == "") { print "Found"; next; } \
			 gsub("[][]",",",$7); split($7, d, "|"); \
			 for (i in d) { \
			  n = split(d[i], f, ","); \
			  if (f[1] == DevDest) {
			   for (i = 2; i <= n; i++) { if (f[i] == Event) {print "Found"; } }
			}}}' $cfg`

		[ "$RES" = "Found" ] && return 0
	done

	return 1
}

function checkAndStartLocalSend_()
{
	local dev
	local optEvent=""
	
	# check if there is any HL7 device configured to the server
	[ ! -f "${HL7_VAR}/device.cfg" ] && return 0
	DEVICES=`awk '/^#/ || /^LOCAL / {next} ($1 != "") {print $1}' ${HL7_VAR}/device.cfg`
	[ -z "$DEVICES" ] && return 0

	[ -n "$EVENT" -a "$EVENT" != "unrecognized" ] && optEvent='-e "'"$EVENT"'"'

	for dev in $DEVICES ; do
		if [ "$dev" = "$DEVICE" ] ; then
			# Device is the originator 
			continue;
		fi

		if ! isEventSupported_ "$dev" ; then
			continue;
		fi

		# add "sendto_$dev" reference to -t HL7MESSAGE -i `basename $FILENAME`
		# reference will be removed by sendMessageTask.so after the message has successfully sent
		$UTILS_HOME/bin/refcount -a "sendto_${dev}" -i "$rcid" -t "HL7MESSAGE" >>$LOG/refcount.error 2>&1

		$TASKD_HOME/createchildtask.sh -p "HL7_LOCAL_SEND_MESSAGE" -t "sh" -c "${HL7_HOME}/bin/sendMessageTask.sh -f \"${FILENAME}\" -d \"${dev}\" -t MSG $optEvent"
	done

	return 0
}


#override run method
function run_()
{
	if [ -n "$ERRMSG" ] ; then
		usage_ "$ERRMSG"
		return 125
	fi

	# add createonly "sendMessage" reference to -t HL7MESSAGE -i `basename $FILENAME`
	# Reference will be removed in clenup_
	$UTILS_HOME/bin/refcount -n "sendMessage" -i "$rcid" -t "HL7MESSAGE" >>$LOG/refcount.error 2>&1

	HasHL7="false"
	[ -f "$IFACE_VAR/messageFlow.cfg" ] && . "$IFACE_VAR/messageFlow.cfg"
	if [ "$HasHL7" != "true" ] ; then
		$LOGGER -p $LOG_FACILITY.info -t "$TAG" "No HL7 configured"
		return 0;
	fi
	
	checkAndStartLocalSend_
	RES=$?
	[ $RES -ne 0 ] && return $RES

	if [ -z "$DeviceTowardHL7" ] ; then
		$LOGGER -p $LOG_FACILITY.info -t "$TAG" "HL7 is connected locally (no device toward HL7)"
		return 0;
	fi

	if [ "$EXCEPT" = "$DeviceTowardHL7" ] ; then
		$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Received from HL7 device $EXCEPT - do not send back"
		return 0;
	fi


	# Upload to the server connected to the RIS
	PEVENT=""
	PDEVICE=""
	[ -n "$EVENT" ] && PEVENT="-a \"-e\" -a \"$EVENT\""
	[ -n "$DEVICE" ] && PDEVICE="-a \"-o\" -a \"$DEVICE\""

	# start Upload task
	$UTILS_HOME/bin/refcount -a "uploadto_$DeviceTowardHL7" -i "$rcid" -t "HL7MESSAGE" >>$LOG/refcount.error 2>&1
	$TASKD_HOME/createchildtask.sh -p "UPLOAD" -t "class" -c "dotcom.Upload -f \"$FILENAME\" -H $DeviceTowardHL7  -R HL7MESSAGE -I \"$rcid\"  -r \"${HL7_HOME}/bin/sendMessage.sh\" $PEVENT $PDEVICE"

	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "Uploading $FILENAME to $DeviceTowardHL7 started"
}

# override print_
function print_()
{
	echo "Send and Proliferate $FILENAME HL7 message [Event: $EVENT; Originated from: $DEVICE]"
}

#
# get display name (for subjob)
#
function get_display_name_()
{
	echo "Checks if message need to be sent to HL7 devices and starts sending"
}

# override cleanup_
function cleanup_()
{
	# TODO: remove "sendMessage" reference from -t HL7MESSAGE -i `basename $FILENAME`
	$UTILS_HOME/bin/refcount -r "sendMessage" -i "$rcid" -t "HL7MESSAGE" >>$LOG/refcount.error 2>&1
}

